# Generated by Django 2.1 on 2018-08-11 18:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spgateway', '0002_generate_slug_for_none'),
    ]

    operations = [
        migrations.AlterField(
            model_name='spgatewaycustomerresponseinfo',
            name='Amt',
            field=models.IntegerField(verbose_name='交易金額'),
        ),
        migrations.AlterField(
            model_name='spgatewaynotifyresponsecredit',
            name='Inst',
            field=models.IntegerField(verbose_name='分期-期別'),
        ),
        migrations.AlterField(
            model_name='spgatewaynotifyresponsecredit',
            name='InstEach',
            field=models.IntegerField(verbose_name='分期-每期金額'),
        ),
        migrations.AlterField(
            model_name='spgatewaynotifyresponsecredit',
            name='InstFirst',
            field=models.IntegerField(verbose_name='分期-首期金額'),
        ),
        migrations.AlterField(
            model_name='spgatewaynotifyresponsecredit',
            name='RedAmt',
            field=models.IntegerField(null=True, verbose_name='紅利折抵後實際金額'),
        ),
        migrations.AlterField(
            model_name='spgatewaynotifyresponsecredit',
            name='TokenUseStatus',
            field=models.IntegerField(verbose_name='信用卡快速結帳使用狀態'),
        ),
        migrations.AlterField(
            model_name='spgatewaynotifyresponseinfo',
            name='Amt',
            field=models.IntegerField(verbose_name='交易金額'),
        ),
    ]



from django.apps import apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.views import generic
from django.views.decorators.csrf import csrf_exempt


from .helpers import decrypt_TradeInfo_TradeSha
from . import models

# Create your views here.


class SpgatewayMixin(object):
    OrderModel = apps.get_model(
        app_label=settings.SPGATEWAY_ORDERMODEL.rsplit('.', 1)[0],
        model_name=settings.SPGATEWAY_ORDERMODEL.rsplit('.', 1)[1],
    )

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(SpgatewayMixin, self).dispatch(request, *args, **kwargs)

    def get_TradeInfo(self, use_json=True):
        MerchantID = self.request.POST.get('MerchantID')
        TradeInfo_encrypted = self.request.POST.get('TradeInfo')
        TradeSha = self.request.POST.get('TradeSha', '')

        return decrypt_TradeInfo_TradeSha(
            HashKey=settings.SPGATEWAY_PROFILE[MerchantID]['HashKey'],
            HashIV=settings.SPGATEWAY_PROFILE[MerchantID]['HashIV'],
            TradeInfo=TradeInfo_encrypted,
            TradeSha=TradeSha,
            use_json=use_json,
        )

    def get_order(self, **kwargs):
        order_obj = None
        order_list = self.OrderModel.objects.filter(SpgatewaySlug=kwargs.get('slug'))
        if order_list:
            order_obj = order_list[0]
        if not order_obj:
            info_list = models.SpgatewayNotifyResponseInfo.objects.filter(MerchantOrderNo=kwargs.get('slug'))
            order_obj = info_list[0].Order if info_list else None
        if not order_obj:
            info_list = models.SpgatewayCustomerResponseInfo.objects.filter(MerchantOrderNo=kwargs.get('slug'))
            order_obj = info_list[0].Order if info_list else None
        if not order_obj:
            raise Http404('No matches the given query.')
        return order_obj

    def get_context_data(self, **kwargs):
        context = {
            'SpgatewayTradeInfo': self.get_TradeInfo(),
        }
        context.update(kwargs)
        return super(SpgatewayMixin, self).get_context_data(**context)

    def get_success_url(self):
        if hasattr(self, 'success_url') and self.success_url:
            return self.success_url.format(**self.object.__dict__)
        elif hasattr(self.object, 'get_absolute_url'):
            return self.object.get_absolute_url()
        else:
            raise ImproperlyConfigured(
                "No URL to redirect to. Provide a success_url.")


class SpgatewayReturnView(SpgatewayMixin, generic.View):
    model = None
    template_name = 'spgateway/response.html'

    def __init__(self, *args, **kwargs):
        self.model = self.OrderModel
        super(SpgatewayReturnView, self).__init__(*args, **kwargs)

    def get_object(self, queryset=None):
        return self.object

    def post(self, request, *args, **kwargs):
        self.request = request
        self.trade_info = self.get_TradeInfo()
        self.object = self.get_order(slug=self.trade_info.get('Result', {}).get('MerchantOrderNo'))

        self.send_notify()

        success_url = self.get_success_url()

        if hasattr(self, 'template_name'):
            result = render(
                self.request, self.template_name,
                {'success_url': success_url, 'TradeInfo': self.trade_info},
            )
        else:
            result = HttpResponseRedirect(success_url)

        return result

    def send_notify(self):
        if hasattr(self.object, 'spgateway_return'):
            self.object.spgateway_return(self.request, self.trade_info)


class SpgatewayResponseViewMixin(SpgatewayMixin):
    def get_base_model(self, trade_info=None):
        raise ImproperlyConfigured('get_base_model not defined')

    def get_additional_model(self, trade_info=None):
        raise ImproperlyConfigured('get_addtional_model not defined')

    def post(self, request, *args, **kwargs):
        self.request = request
        self.trade_info = self.get_TradeInfo()
        self.object = self.get_order(slug=self.trade_info.get('Result', {}).get('MerchantOrderNo'))

        base_model = self.get_base_model(self.trade_info)
        additional_model = self.get_additional_model(self.trade_info)

        object = base_model()
        additional_object = additional_model(Info=object)

        trade_info_result = self.trade_info.get('Result', {})
        trade_info_result_keys = trade_info_result.keys()
        for each_object in [object, additional_object]:
            for each_field in each_object._meta.get_fields():
                if each_field.name in trade_info_result_keys:
                    setattr(each_object, each_field.name, trade_info_result[each_field.name])

        object.Order = self.object
        object.save()
        additional_object.Info = object
        additional_object.save()

        status = self.trade_info['Status']
        if status != 'SUCCESS':
            if self.object.SpgatewaySlug == self.trade_info.get('Result', {}).get('MerchantOrderNo'):
                self.object.regenerate_slug()

        self.send_notify()

        success_url = self.get_success_url()

        if hasattr(self, 'template_name'):
            result = render(self.request, self.template_name, {'success_url': success_url, 'TradeInfo': self.trade_info})
        else:
            result = HttpResponseRedirect(success_url)

        return result


class SpgatewayNotifyView(SpgatewayResponseViewMixin, generic.View):
    template_name = 'spgateway/response.html'

    def get_base_model(self, trade_info=None):
        return models.SpgatewayNotifyResponseInfo

    def get_additional_model(self, trade_info=None):
        if trade_info is None:
            if self.trade_info:
                trade_info = self.trade_info
            else:
                trade_info = self.trade_info = self.get_TradeInfo()

        payment_type = trade_info.get('Result', {}).get('PaymentType')
        if payment_type == 'CREDIT':
            return models.SpgatewayNotifyResponseCredit
        elif payment_type == 'WEBATM':
            return models.SpgatewayNotifyResponseATM
        elif payment_type == 'VACC':
            return models.SpgatewayNotifyResponseATM
        elif payment_type == 'CVS':
            return models.SpgatewayNotifyResponseCVS
        elif payment_type == 'BARCODE':
            return models.SpgatewayNotifyResponseBarcode
        else:
            raise Exception('Unknown payment_type')

    def send_notify(self):
        if hasattr(self.object, 'spgateway_notify'):
            self.object.spgateway_notify(self.request, self.trade_info)


class SpgatewayCustomerView(SpgatewayResponseViewMixin, generic.View):
    template_name = 'spgateway/response.html'

    def get_base_model(self, trade_info=None):
        return models.SpgatewayCustomerResponseInfo

    def get_additional_model(self, trade_info=None):
        if trade_info is None:
            if self.trade_info:
                trade_info = self.trade_info
            else:
                trade_info = self.trade_info = self.get_TradeInfo()

        payment_type = trade_info.get('Result', {}).get('PaymentType')
        if payment_type == 'VACC':
            return models.SpgatewayCustomerResponseVACC
        elif payment_type == 'CVS':
            return models.SpgatewayCustomerResponseCVS
        elif payment_type == 'BARCODE':
            return models.SpgatewayCustomerResponseBarcode
        else:
            raise Exception('Unknown payment_type')

    def send_notify(self):
        if hasattr(self.object, 'spgateway_customer'):
            self.object.spgateway_customer(self.request, self.trade_info)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-21 16:54
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion
import spgateway.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SpgatewayCustomerResponseBarcode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('Barcode_1', models.CharField(max_length=20, verbose_name='第一段條碼')),
                ('Barcode_2', models.CharField(max_length=20, verbose_name='第二段條碼')),
                ('Barcode_3', models.CharField(max_length=20, verbose_name='第三段條碼')),
            ],
        ),
        migrations.CreateModel(
            name='SpgatewayCustomerResponseCVS',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('CodeNo', models.CharField(max_length=30, verbose_name='繳費代碼')),
            ],
        ),
        migrations.CreateModel(
            name='SpgatewayCustomerResponseInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('MerchantID', models.CharField(max_length=15, verbose_name='商店代號')),
                ('Amt', models.IntegerField(max_length=10, verbose_name='交易金額')),
                ('TradeNo', models.CharField(max_length=20, verbose_name='智付通交易序號')),
                ('MerchantOrderNo', models.CharField(max_length=20, verbose_name='商店訂單編號')),
                ('PaymentType', models.CharField(max_length=10, verbose_name='支付方式')),
                ('ExpireDate', models.DateTimeField(verbose_name='繳費截止日期')),
            ],
            bases=(spgateway.models.SpgatewayResponseMixin, models.Model),
        ),
        migrations.CreateModel(
            name='SpgatewayCustomerResponseVACC',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('BankCode', models.CharField(max_length=10, verbose_name='金融機構代碼')),
                ('CodeNo', models.CharField(max_length=30, verbose_name='繳費代碼')),
                ('Info', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spgateway.SpgatewayCustomerResponseInfo')),
            ],
        ),
        migrations.CreateModel(
            name='SpgatewayNotifyResponseATM',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('PayBankCode', models.CharField(max_length=10, verbose_name='付款人金融機構代碼')),
                ('PayerAccount5Code', models.CharField(max_length=5, verbose_name='付款人金融機構帳號末五碼')),
            ],
        ),
        migrations.CreateModel(
            name='SpgatewayNotifyResponseBarcode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('Barcode_1', models.CharField(max_length=20, verbose_name='第一段條碼')),
                ('Barcode_2', models.CharField(max_length=20, verbose_name='第二段條碼')),
                ('Barcode_3', models.CharField(max_length=20, verbose_name='第三段條碼')),
                ('PayStore', models.CharField(max_length=8, verbose_name='繳費超商')),
            ],
        ),
        migrations.CreateModel(
            name='SpgatewayNotifyResponseCredit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('RespondCode', models.CharField(max_length=5, verbose_name='金融機構回應碼')),
                ('Auth', models.CharField(max_length=6, verbose_name='授權碼')),
                ('Card6No', models.CharField(max_length=6, verbose_name='卡號前六碼')),
                ('Card4No', models.CharField(max_length=4, verbose_name='卡號末四碼')),
                ('Inst', models.IntegerField(max_length=10, verbose_name='分期-期別')),
                ('InstFirst', models.IntegerField(max_length=10, verbose_name='分期-首期金額')),
                ('InstEach', models.IntegerField(max_length=10, verbose_name='分期-每期金額')),
                ('ECI', models.CharField(max_length=2, verbose_name='ECI 值')),
                ('TokenUseStatus', models.IntegerField(max_length=1, verbose_name='信用卡快速結帳使用狀態')),
                ('RedAmt', models.IntegerField(max_length=5, null=True, verbose_name='紅利折抵後實際金額')),
            ],
        ),
        migrations.CreateModel(
            name='SpgatewayNotifyResponseCVS',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('CodeNo', models.CharField(max_length=30, verbose_name='繳費代碼')),
            ],
        ),
        migrations.CreateModel(
            name='SpgatewayNotifyResponseInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('MerchantID', models.CharField(max_length=15, verbose_name='商店代號')),
                ('Amt', models.IntegerField(max_length=10, verbose_name='交易金額')),
                ('TradeNo', models.CharField(max_length=20, verbose_name='智付通交易序號')),
                ('MerchantOrderNo', models.CharField(max_length=20, verbose_name='商店訂單編號')),
                ('PaymentType', models.CharField(max_length=10, verbose_name='支付方式')),
                ('RespondType', models.CharField(max_length=10, verbose_name='回傳格式')),
                ('PayTime', models.DateTimeField(verbose_name='支付完成時間')),
                ('IP', models.CharField(max_length=15, verbose_name='交易 IP')),
                ('EscrowBank', models.CharField(max_length=10, verbose_name='履保銀行')),
                ('Order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.SPGATEWAY_ORDERMODEL, verbose_name='訂單')),
            ],
            bases=(spgateway.models.SpgatewayResponseMixin, models.Model),
        ),
        migrations.AddField(
            model_name='spgatewaynotifyresponsecvs',
            name='Info',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spgateway.SpgatewayNotifyResponseInfo'),
        ),
        migrations.AddField(
            model_name='spgatewaynotifyresponsecredit',
            name='Info',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spgateway.SpgatewayNotifyResponseInfo'),
        ),
        migrations.AddField(
            model_name='spgatewaynotifyresponsebarcode',
            name='Info',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spgateway.SpgatewayNotifyResponseInfo'),
        ),
        migrations.AddField(
            model_name='spgatewaynotifyresponseatm',
            name='Info',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spgateway.SpgatewayNotifyResponseInfo'),
        ),
        migrations.AddField(
            model_name='spgatewaycustomerresponsecvs',
            name='Info',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spgateway.SpgatewayCustomerResponseInfo'),
        ),
        migrations.AddField(
            model_name='spgatewaycustomerresponsebarcode',
            name='Info',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='spgateway.SpgatewayCustomerResponseInfo'),
        ),
    ]



from django import forms
from django.conf import settings


class SpgatewayForm(forms.Form):
    MerchantID = forms.CharField(max_length=15, widget=forms.HiddenInput())
    TradeInfo = forms.CharField(widget=forms.HiddenInput())
    TradeSha = forms.CharField(widget=forms.HiddenInput())
    Version = forms.CharField(max_length=5, initial='1.4', widget=forms.HiddenInput())
    action = 'https://ccore.spgateway.com/MPG/mpg_gateway' if settings.DEBUG else 'https://core.spgateway.com/MPG/mpg_gateway'



# Generated by Django 2.2.1 on 2019-05-24 02:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('estore', '0012_order_spgatewayslug'),
        ('spgateway', '0003_auto_20180811_1809'),
    ]

    operations = [
        migrations.AddField(
            model_name='spgatewaycustomerresponseinfo',
            name='Order',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, to='estore.Order', verbose_name='訂單'),
            preserve_default=False,
        ),
    ]
